<?php
/**
 * User: nester_a
 * Date: 29.10.13
 */

namespace Webex\Repository;

use LMS\Tools\Conference\Repository;
use LMS\Libraries\Exception\Http;
use LMS\Libraries\Exception\Database;
use LMS\Libraries\Date;

class Booking extends Repository\Booking
{

	/**
	 * @var \JLMSXML
	 */
	private $xmlService = null;

	private $webexRemoteBookingRepository;
	private $webexBookingRepository;
	

    public function __construct($db)
    {
		$this->xmlService = \JLMSFactory::getXMLParser();
		$this->webexBookingRepository = new WebexBooking(array());
		$this->webexRemoteBookingRepository = new Remote\Booking(array());
        parent::__construct($db);
    }


    /**
     * Function to get list of available conferences at current time
     *
     * @param int $courseID
     * @return mixed
     */
    public function getAvailable($courseID, $timeOffset, $withTeacherNames=false)
    {
        $join = $withTeacherNames ? ' LEFT JOIN #__users as b ON a.teacher_id = b.id' : '';
        $select = $withTeacherNames ? ', b.username as teacher_name' : '';

        $currentTimestamp = ( time() + $timeOffset * 60 * 60 );
        $query = "SELECT a.* $select, webex.meeting_id AS meeting_id, webex.join_url AS join_url
                    FROM #__lms_conference_period a
                        $join
                        LEFT JOIN #__lms_conference_webex webex ON webex.conf_id=a.p_id
                        WHERE
                            a.course_id='" .$courseID."'
                        AND
                            a.from_time <= ".$currentTimestamp ."
                        AND
                            a.to_time >= ".$currentTimestamp;

        $this->db->setQuery( $query );
        return $this->db->loadObjectList();
    }

    /**
     * @param $bookedConferenceID
     * @return mixed
     */
    public function getMeetingByConferenceID($bookedConferenceID)
    {
        $query = "SELECT *
                    FROM
                        #__lms_conference_webex WHERE conf_id=$bookedConferenceID";
        $this->db->setQuery($query);
        return $this->db->loadObject();
    }

    /**
     * @param $uniqueMeetingIDs
     * @return mixed
     */
    public function getDeletedFromGTMService($uniqueMeetingIDs)
    {
        $query = "SELECT conf_id
                    FROM #__lms_conference_webex
                    WHERE
                        unique_meeting_id NOT IN ('".implode('\',\'', $uniqueMeetingIDs)."')";
        $this->db->setQuery($query);
        return $this->db->loadObjectList();
    }

    /**
     * Return count of booked conferences filtered by student id and teacher id
     *
     * @param int $courseID
     * @param int $teacherID
     * @param int $studentID
     * @return int
     */
    public function count($courseID, $teacherID=false, $studentID=false)
    {
        $additionalJoin = $studentID ? (" LEFT JOIN #__lms_conference_usr as b ON c.p_id=b.p_id") : '';
        $additionalJoin.= ' JOIN #__lms_conference_webex as webexc ON webexc.conf_id = a.p_id';

        $additionalWhere = $teacherID ? " AND a.teacher_id = $teacherID " : '';
        $additionalWhere.= $studentID ? " AND a.p_id = b.p_id AND b.user_id = $studentID" : "";

        $query = "SELECT COUNT(distinct a.p_id)
                        FROM #__lms_conference_period as a
                        $additionalJoin
                                LEFT JOIN #__users as c ON a.teacher_id = c.id
                        WHERE course_id='".$courseID."'" . $additionalWhere;

        $this->db->setQuery($query);
        return $this->db->LoadResult();
    }


	/**
	 * @param array $data
	 * @return bool|mixed
	 * @throws \LMS\Libraries\Exception\Http\FailedRequest
	 * @throws \LMS\Libraries\Exception\Database\Error
	 */
	public function save($data)
	{
		$meetingData = $this->webexRemoteBookingRepository->save($data);
		if($meetingData === false){
			throw new Http\FailedRequest('HTTP error - Sending request to WebEx API failed');
		}

		$conferenceID = parent::save($data) ? $this->lastInsertedID() : false;
		if($conferenceID == false){
			throw new Database\Error('Error occurred while saving in LMS table');
		}

		$dataToBeSaved = new \stdClass();
		$dataToBeSaved->meeting = $meetingData;
		$dataToBeSaved->conferenceID = $conferenceID;

		if(!$this->webexBookingRepository->save($dataToBeSaved)){
			throw new Database\Error('Error occurred while saving in WebEx table');
		}
		return true;
	}

	/**
	 * @param array $data
	 * @return bool|mixed
	 * @throws \LMS\Libraries\Exception\Http\FailedRequest
	 */
	public function saveMultiple($data)
	{
		foreach($data as $item){
			$meetingData = $this->webexRemoteBookingRepository->save($item);
			if($meetingData == false){
				throw new Http\FailedRequest('HTTP error - Sending request to GoToMeeting API failed');
			}

			$conferenceID = parent::save($item) ? $this->lastInsertedID() : false;
			if($conferenceID == false){
				return false;
			}

			$dataToBeSaved = $meetingData[0];
			$dataToBeSaved->conferenceID = $conferenceID;

			$this->webexBookingRepository->save((array)$dataToBeSaved);
		}

		return true;
	}

	/**
	 * @param array $where
	 * @return bool|mixed
	 * @throws \LMS\Libraries\Exception\Http\FailedRequest
	 * @throws \LMS\Libraries\Exception\Database\Error
	 */
	public function delete($where)
	{
		$bookedConferenceIDs = $where['p_id'];

		$conferenceDeletedFromGTM = array();

		foreach($bookedConferenceIDs as $conferenceID){
			$meeting = $this->getMeetingByConferenceID($conferenceID);
			if($this->webexRemoteBookingRepository->delete($meeting->meeting_id) !== false){
				$conferenceDeletedFromGTM[] = $conferenceID;
			}else{
				$httpErrorOccurred = true;
			}
		}

		if(!$this->webexBookingRepository->delete(array('conf_id' => $conferenceDeletedFromGTM))){
			throw new Database\Error('Error occurred while deleting from GTM table');
		}

		if(!parent::delete(array('p_id' => $conferenceDeletedFromGTM))){
			throw new Database\Error('Error occurred while deleting from LMS table');
		}

		if(isset($httpErrorOccurred)){
			throw new Http\FailedRequest('HTTP error - Sending request to GoToMeeting API failed');
		}

		return true;
	}

	/**
	 * @param array $params
	 * @param array $where
	 * @return bool|mixed
	 * @throws \LMS\Libraries\Exception\Http\FailedRequest
	 */
	public function update($params, $where)
	{
		$meeting = $this->getMeetingByConferenceID($where['p_id']);
		if(is_null($meeting)){
			return false;
		}

		if($this->webexRemoteBookingRepository->update($meeting, $params) === false){
			throw new Http\FailedRequest('HTTP error - Sending request to GoToMeeting API failed');
		}

		return parent::update($params, $where);
	}




	protected function updateRemoteService($bookedConferenceID, $params)
	{
		$date = new DateTime();
		$params['duration'] = ($params['daytime_to'] - $params['daytime_from']) / 60;
		$params['timezoneID'] = $date->getOffset() / 3600;
		$params['start_date'] = date("m/d/Y H:i:s", $params['daytime_from']);

		$meeting = $this->getMeetingByConferenceID($bookedConferenceID);
		$params['meetingKey'] = $meeting->meeting_id;

		$schema = RequestHelper::loadXMLSchema(__DIR__ . '/../schemas/meeting/update.xml.phtml', $params);
		$response = RequestHelper::sendJSONRequest('https://apidemoeu.webex.com/WBXService/XMLService', $schema, 'POST',  array('Content-Type: text/xml'));

		$meetingData = new SimpleXMLElement($response);
		$result = $meetingData->xpath('//serv:result');
		return $result == 'SUCCESS';
	}




	protected function prepareFilter($query, &$filterOptions)
	{
		$query = parent::prepareFilter($query, $filterOptions);
		$query->innerJoin($this->webexBookingRepository->getTable()->getTableName() . ' as webexc ON webexc.conf_id = booking.p_id');
		return $query;
	}

	/**
	 * @param $params
	 * @return array
	 */
	protected function prepareDataRemoteService($params)
	{
		return array(
			'subject' => $params['p_name'],
			'starttime' =>  $params['fromTime'],
			'endtime' => $params['toTime'],
			'passwordrequired' => (string)($params['c_public'] == 0),
			'conferencecallinfo' => $params['p_description'],
			'timezonekey' => "",
			'meetingtype' => 'Scheduled'
		);

	}

	protected function prepareUTCTimes($params)
	{
		$times = array();

		$times['fromTime'] = gmdate('Y-m-d\TH:i:s', $params['daytime_from']);
		$times['toTime'] = gmdate('Y-m-d\TH:i:s', $params['daytime_to']);

		return $times;
	}

}