<?php
/**
 * User: nester_a
 * Date: 25.06.14
 */

namespace LMS\Libraries\Database;

use LMS\Libraries\ServiceLocator;

class Object extends \stdClass
{
	protected $_object_id;

	function __construct()
	{
		$this->_object_id = 'object_' . time().rand(0,10000).rand(0,10000).rand(0,10000);
	}

	public function __get($name)
	{
		$name = strtolower($name);

		$relations = $this->getTable()->getRelations();
		if(!isset($relations[$name])){
			return null;
		}

		$relation = $relations[$name];
		$db = $this->getTable()->getDbo();

		$query = $db->getQuery(true);
		$query->from($relation['table']);

		$query->select($db->quoteName($relation['table']->getTableAlias()) . '.*');
		$query->where($this->processCondition($relation));

		$db->setQuery($query);
		return $relation['capacity'] == \LMS\Libraries\Base\Table::MANY ? $db->loadObjectList() : $db->loadObject();
	}

	function __call($name, $arguments)
	{
		if(strpos($name, 'get') !== 0){
			throw new \InvalidArgumentException('There is no method ' . $name . ' in class '.__CLASS__);
		}

		$relationName = strtolower(substr($name, 3));

		$relations = $this->getTable()->getRelations();
		if(!isset($relations[$relationName])){
			return null;
		}

		$relation = $relations[$relationName];
		$db = $this->getTable()->getDbo();

		$query = $db->getQuery(true);
		$query->from($relation['table']);
		$query->select($db->quoteName($relation['table']->getTableAlias()) . '.*');
		$query->where($this->processCondition($relation));

		if(!empty($arguments)){
			foreach($arguments[0] as $arg => $value){
				if(is_array($value)){
					$query->where($db->quoteName($relation['table']->getTableAlias()) . '.' .$arg.' IN ("' . implode('","', $value) . '")');
				}else{
					$query->where($db->quoteName($relation['table']->getTableAlias()) . '.' .$arg.' = '.$db->quote($value));
				}
			}
		}

		$db->setQuery($query);
		return $relation['capacity'] == \LMS\Libraries\Base\Table::MANY ? $db->loadObjectList() : $db->loadObject();
	}


	public function setTable(\LMS\Libraries\Base\Table $table)
	{
		$registry = ServiceLocator::get('registry');
		return $registry->set($this->_object_id, $table);
	}

	/**
	 * @return \LMS\Libraries\Base\Table
	 */
	public function getTable()
	{
		$registry = ServiceLocator::get('registry');
		return $registry->get($this->_object_id);

	}

	private function processCondition($relation)
	{
		$db = $this->getTable()->getDbo();
		$condition = $relation['condition'];
		preg_match_all('/{([0-9A-Za-z_\-\.]+)}/', $condition, $matches);
		foreach($matches[1] as $field){
			$parts = explode('.', $field);
			if($parts[0] == 'parent'){
				$fieldName = $this->getTable()->getField($parts[1]);
				$condition = str_replace('{'.$field.'}', $this->getTable()->getDbo()->quote($this->$fieldName), $condition);
			}else{
				$fieldName = $db->quoteName($relation['table']->getTableAlias()) . '.' . $relation['table']->getField($parts[1]);
				$condition = str_replace('{'.$field.'}', $fieldName, $condition);
			}
		}
		return $condition;
	}

}