<?php
/**
 * User: nester_a
 * Date: 05.11.13
 */

namespace LMS\Tools\Conference\Response;

use LMS\Libraries\Message;
use LMS\Libraries\ServiceLocator;
use Phalcon\DI\Service;

class Handler
{
    private $session;

	/**
	 * @var Message\Manager
	 */
	private $msgManager;

	/**
	 * @var \LMS\Libraries\Routing\Handler
	 */
	private $router;

    public function __construct($diContainer)
    {
        $this->session = $diContainer['session'];
		$this->msgManager = $diContainer['msgManager'];
		$this->router = ServiceLocator::get('conference_router');
    }

	public function handle($results)
	{
		$result =  isset($results[0]) ? $results[0] : null;
		if(is_null($result) || !($result instanceof Response ) ){
			return new \LMS\Widgets\Note(array('message' => 'There\'s no enabled conference plugins!', 'type' => 'warning'));
		}

		$method = 'handle' . $result->getType();
		return $this->$method($result);

	}

    private function handleContent(Response $response)
    {
        if($response->getStatus() == Response::STATUS_ERROR){
            throw new \Exception($response->getContent());
        }
        return $response->getContent();
    }

	private function handleOperation(Response $response)
    {
		$this->msgManager->success($response->getContent());
		$this->router->redirectBack();
    }

	private function handleData(Response $response)
    {
        while( @ob_end_clean() );
        echo $response->getContent();
        exit();
    }

	private function handleFile(Response $response)
    {
        while( @ob_end_clean() );

        require_once(_JOOMLMS_FRONT_HOME . DS . "includes" . DS . 'jlms_docs_process.php');
        $mimeType = 'application/octet-stream';
        $filename = $response->getContent();
        if (preg_match('/\.(.*)$/U', $filename, $match)) {
            $mimeInfo = JLMS_GetMimeType($match[1]);
            $mimeType = $mimeInfo['type'];
        }

        $headers = $response->getHeaders();
        header('Content-Type: ' . $mimeType);
        header('Content-Length: ' . $headers['Content-Length']);
        header("Content-disposition: " . $headers['Content-disposition']);




        readfile($filename);
        exit();
    }

	private function handleLMSFile(Response $response)
    {
        while( @ob_end_clean() );
        JLMS_downloadFile($response->getContent(), 'com_joomla_lms', '', true, $response->getHeaders());
        exit();
    }

	private function handleXMLData(Response $response)
    {
        while( @ob_end_clean() );
        echo $response->getContent();
        exit();
    }

	private function handleJSONData(Response $response)
    {
        while( @ob_end_clean() );
        echo $response->getContent();
        exit();
    }
}